/*!
 * @file        apm32f00x_iwdt.c
 *
 * @brief       This file contains all the functions for the IWDT peripheral         
 *
 * @version     V1.0.0
 *
 * @date        2019-10-16
 *
 */

#include "apm32f00x_iwdt.h"

/** @addtogroup Peripherals_Library Standard Peripheral Library  
  @{
*/

/** @addtogroup IWDT_Driver IWDT Driver
  @{
*/

/** @addtogroup IWDT_Fuctions Fuctions
  @{
*/

/*!
 * @brief       Enable write access to Divider and Counter Reload registers
 *
 * @param       None
 *
 * @retval      None
 *
 * @note       
 */
void IWDT_EnableWriteAccess(void)
{
    IWDT->KEYWORD = IWDT_KEY_ACCESS;
}

/*!
 * @brief       Disable write access to Divider and Counter Reload registers
 *
 * @param       None
 *
 * @retval      None
 *
 * @note       
 */
void IWDT_DisableWriteAccess(void)
{
    IWDT->KEYWORD = 0;
}

/*!
 * @brief       Divider configuration
 *
 * @param       div:    Specifies the divider
 *
 * @retval      None
 *
 * @note       
 */
void IWDT_ConfigDivider(IWDT_DIV_T div)
{
    IWDT->DIV = (uint32_t)div;
}


/*!
 * @brief       Set counter reload value
 *
 * @param       reload: Specifies the reload value
 *
 * @retval      None
 *
 * @note       
 */
void IWDT_SetCntReload(uint8_t reload)
{
    IWDT->CNTRLD = (uint32_t)reload;
}

/*!
 * @brief       Refresh IWDT       
 *
 * @param       None
 *
 * @retval      None
 *
 * @note       
 */
void IWDT_Refresh(void)
{
    IWDT->KEYWORD = IWDT_KEY_REFRESH;
}

/*!
 * @brief       Enable IWDT
 *
 * @param       None
 *
 * @retval      None
 *
 * @note       
 */
void IWDT_Enable(void)
{
    IWDT->KEYWORD = IWDT_KEY_ENABLE;
}

/**@} end of group IWDT_Fuctions*/
/**@} end of group IWDT_Driver */
/**@} end of group Peripherals_Library*/  

